
#############################################################################
# Copyright (C) Labomedia May 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franproplin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################

import GameLogic as gl
import socket
from OSC3.OSC import decodeOSC
from time import sleep
from math import sqrt, atan, tan

# init global var
ip = gl.ip
port = gl.port_in
buffer_size = 1024

data = [] # osc data with modif along the script
players_data = [] # list of list

# Get controller and owner
controller = gl.getCurrentController()
owner = controller.owner
# Get all scenes
scenes = gl.getSceneList()

def main():
    # For all scenes, all objets, all prop
    # Get bat and ball
    for scen in scenes:
        for obj in scen.objects:
            # always
            if str(obj) == "GetOSC":  getosc = obj
            if str(obj) == "SendOSC":  sendosc = obj
            # level 1
            if str(obj) == "bat10": bat10 = obj
            if str(obj) == "bat11": bat11 = obj
            if str(obj) == "ball1": ball1 = obj
            # level 2
            if str(obj) == "bat20": bat20 = obj
            if str(obj) == "bat21": bat21 = obj
            if str(obj) == "ball2": ball2 = obj
            if str(obj) == "goal20": goal20 = obj
            if str(obj) == "goal21": goal21 = obj
            # level 3
            if str(obj) == "bat30": bat30 = obj
            if str(obj) == "bat31": bat31 = obj
            if str(obj) == "bat32": bat32 = obj
            if str(obj) == "ball3": ball3 = obj
            if str(obj) == "goal30": goal30 = obj
            if str(obj) == "goal31": goal31 = obj
            if str(obj) == "goal32": goal32 = obj
    data = get_osc()
    i_quit(data) # I quit if /quit message in data

    global players_data
    players_data = set_players_data(data) # always list of list

    # my position in list
    who = who_are_you(players_data)
    level = len(players_data)
    if level == 0:
        getosc["Text"] = "Problem with Network or Server"
    else:
        getosc["Text"] = "Player number =  " + str(level)
    getosc["level"] = level

    # ip, goal0, goal1, goal2, bat0x, bat0y, bat1x, bat1y, bat2x, bat2y, ballx, bally
    #  0    1       2       3   4       5       6      7    8       9       10      11
    # Set ball position if I am not master
    # The first player in list is always the master
    # If one player, always master
    if who == 1 or who == 2:
        if level == 2:
            # disable ball rigid body with prop master=False in ball
            ball2["master"] = False
            ball2.localPosition = [players_data[0][10], players_data[0][11], 1]
        if level == 3:
            # disable ball rigid body with prop master=False in ball
            ball3["master"] = False
            ball3.localPosition = [players_data[0][10], players_data[0][11], 1]

    # Set automatic bat with one player
    if level == 1:
        y = 0.6*ball1.localPosition[1]
        bat11.localPosition = [9.5, y, 1]

        # Recenter ball if Reset
        if sendosc["reset"] == True:
            ball1.localPosition = [0, 0, 1]

    # set bat and goal
    if level == 2:
        if who == 0:
            # set prop who
            bat20["who"] = 1 # bat20 move with keyboard
            bat21["who"] = 0
            # Set bat of the second player: he is the second in the list: players_data[1]
            bat21.localPosition = [players_data[1][6], players_data[1][7], 1]
            # if goal21 = -1, you win
            # if goal20 = -1, you loose
            if goal20["score"] == 0:
                goal20["score"] = -1 # you loose
                ball2.localPosition = [0, 0, 1]
            if goal21["score"] == 0:
                goal21["score"] = -1 # you win
                ball2.localPosition = [0, 0, 1]

        if who == 1:
            # set prop who
            bat20["who"] = 0
            bat21["who"] = 1 # bat21 move with keyboard
            # Set bat of the first player: he is the first in the list: players_data[0]
            bat20.localPosition = [players_data[0][4], players_data[0][5], 1]
            goal20["score"] = players_data[0][1]
            goal21["score"] = players_data[0][2]
            if goal20["score"] == 0:
                goal21["score"] = -1 # you win
                ball2.localPosition = [0, 0, 1]
            if goal21["score"] == 0:
                goal20["score"] = -1 # you loose
                ball2.localPosition = [0, 0, 1]

        # Recenter ball if Reset
        if sendosc["reset"] == True:
            ball2.localPosition = [0, 0, 1]

    # set bat and goal
    if level == 3:
        #print(scenes)
        ##scenes[0].suspend()
        ##sleep(10)
        ##scenes[0].restart()
        if who == 0:
            bat30["who"] = 1
            bat31["who"] = 0
            bat32["who"] = 0
            # bat31 is the second player in the list
            bat31.localPosition = [players_data[1][6], players_data[1][7], 1]
            # bat32 is the third player in the list
            bat32.localPosition = [players_data[2][8], players_data[2][9], 1]
            # if goal21 = -1, you win
            # if goal20 = -1, you loose
            if goal30["score"] == 0:
                goal30["score"] = -1 # you loose
                ball3.localPosition = [0, 0, 1]
            if goal31["score"] == 0:
                goal31["score"] = -1 # you win
                ball3.localPosition = [0, 0, 1]
        if who == 1:
            bat30["who"] = 0
            bat31["who"] = 1
            bat32["who"] = 0
            # bat30 is the first player in the list
            bat30.localPosition = [players_data[0][4], players_data[0][5], 1]
            # bat32 is the third player in the list
            bat32.localPosition = [players_data[2][8], players_data[2][9], 1]
            goal30["score"] = players_data[0][1]
            goal31["score"] = players_data[0][2]
            goal32["score"] = players_data[0][3]
            # if goal21 = -1, you win
            # if goal20 = -1, you loose
            if goal30["score"] == 0:
                goal30["score"] = -1 # you loose
                ball3.localPosition = [0, 0, 1]
            if goal31["score"] == 0:
                goal31["score"] = -1 # you win
                ball3.localPosition = [0, 0, 1]
        if who == 2:
            bat30["who"] = 0
            bat31["who"] = 0
            bat32["who"] = 1
            # bat30 is the first player in the list
            bat30.localPosition = [players_data[0][4], players_data[0][5], 1]
            # bat31 is the third player in the list
            bat31.localPosition = [players_data[1][6], players_data[1][7], 1]
            goal30["score"] = players_data[0][1]
            goal31["score"] = players_data[0][2]
            goal32["score"] = players_data[0][3]
            # if goal21 = -1, you win
            # if goal20 = -1, you loose
            if goal30["score"] == 0:
                goal30["score"] = -1 # you loose
                ball3.localPosition = [0, 0, 1]
            if goal31["score"] == 0:
                goal31["score"] = -1 # you win
                ball3.localPosition = [0, 0, 1]

        # Recenter ball if Reset
        if sendosc["reset"] == True:
            ball3.localPosition = [0, 0, 1]

    print_usefull_var((players_data, who))

def who_are_you(players_data):
    # where is my ip ?
    who = 0
    for i in range(len(players_data)):
        if players_data[i][0] == gl.local_ip:
            who = i
    return who

def set_players_data(data):
    # a list of data of all players
    if '/for_all_players' in data :
        # Cut /for_all_players and ,ssffiissffii
        data = data[2:]
        if len(data) % 12 == 0 : # verif pour qu il ne manque pas de data
            players_data = chunks(data, 12) # division en sous liste
        else:
            # data is corumpted like politician
            print("data corumpted")
            players_data = []
    else:
        print("No msg /for_all_players")
        players_data = []
    return players_data

def get_osc():
    global data
    # Connect Blender
    if not gl.connected :
        # if socket exist and not close
        try:
            gl.sock.close()
        except:
            pass

        gl.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            gl.sock.bind((ip, port))
            gl.sock.setblocking(0)
            gl.sock.settimeout(0.005)
            gl.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, buffer_size)
            print('Get OSC on IP={} Port={} Buffer={}'.format(ip, port, buffer_size))
            gl.connected = True
        except:
            print('No connected to IP={} Port={}'.format(ip, port))
            pass

    # If Blender connected
    if gl.connected :
        try:
            raw_data = gl.sock.recv(buffer_size)
            #print(raw_data)
            data = decodeOSC(raw_data)
            #print(data)
        except socket.error:
            pass
    return data

def disconnect():
    gl.sock.close()

def chunks(l, n):
    # coupe en n listes de liste de len l
    return [l[i:i+n] for i in range(0, len(l), n)]

def print_usefull_var(usefull_var):
    # print every 60 frames
    gl.tp += 1
    if gl.tp == 60:
        print(usefull_var)
        gl.tp = 0

def i_quit(data):
    # very bad atchtung
    global players_data
    if '/quit' in data :
        gl.sock.close()

main()

##print(scenes)
##scenes[0].suspend()
##sleep(10)
##scenes[0].restart()
